@@ -46,6 +46,12 @@ options:
         in the generated documentation. The flags ``:inherited-members:`` or
         ``:no-inherited-members:`` allows overrriding this global setting.
 
+    * ``:sort:``
+        If the module contains ``__all__``, sort the module's objects
+        alphabetically (if ``__all__`` is not present, the objects are found
+        using `dir`, which always gives a sorted list).
+
+
 This extension also adds three sphinx configuration options:
 
 * ``automodsumm_writereprocessed``
@@ -68,6 +74,11 @@ This extension also adds three sphinx configuration options:
     methods like ``__getitem__`` and ``__setitem__``. Defaults to
     ``['__init__', '__call__']``.
 
+* ``automodsumm_properties_are_attributes``
+    Should be a bool and if ``True`` properties are treated as attributes in the
+    documentation meaning that no property specific documentation is generated.
+    Defaults to ``True``.
+
 .. _sphinx.ext.autosummary: http://sphinx-doc.org/latest/ext/autosummary.html
 .. _autosummary: http://sphinx-doc.org/latest/ext/autosummary.html#directive-autosummary
 
@@ -92,17 +103,21 @@ package. It accepts no options.
 import inspect
 import os
 import re
+import dataclasses
 
+import sphinx
+from docutils.parsers.rst.directives import flag
+from packaging.version import Version
 from sphinx.util import logging
 from sphinx.ext.autosummary import Autosummary
 from sphinx.ext.inheritance_diagram import InheritanceDiagram, InheritanceGraph, try_import
-from docutils.parsers.rst.directives import flag
 
-from .utils import find_mod_objs, cleanup_whitespace
+from .utils import find_mod_objs, cleanup_whitespace, SPHINX_LT_9
 
 __all__ = ['Automoddiagram', 'Automodsumm', 'automodsumm_to_autosummary_lines',
            'generate_automodsumm_docs', 'process_automodsumm_generation']
 logger = logging.getLogger(__name__)
+SPHINX_LT_8_2 = Version(sphinx.__version__) < Version("8.2")
 
 
 def _str_list_converter(argument):
@@ -130,6 +145,7 @@ class Automodsumm(Autosummary):
     option_spec['inherited-members'] = flag
     option_spec['no-inherited-members'] = flag
     option_spec['noindex'] = flag
+    option_spec['sort'] = flag
 
     def run(self):
         env = self.state.document.settings.env
@@ -138,7 +154,7 @@ class Automodsumm(Autosummary):
         nodelist = []
 
         try:
-            localnames, fqns, objs = find_mod_objs(modname)
+            localnames, fqns, objs = find_mod_objs(modname, sort='sort' in self.options)
         except ImportError:
             logger.warning("Couldn't import module " + modname)
             return []
@@ -202,10 +218,10 @@ class Automodsumm(Autosummary):
             self.content = []
 
     def get_items(self, names):
-        try:
-            self.bridge.genopt['imported-members'] = True
-        except AttributeError:  # Sphinx < 4.0
-            self.genopt['imported-members'] = True
+
+        if SPHINX_LT_9:
+            self.bridge.genopt.imported_members = True
+
         return Autosummary.get_items(self, names)
 
 
@@ -260,15 +276,24 @@ class Automoddiagram(InheritanceDiagram):
 
 old_generate_dot = InheritanceGraph.generate_dot
 
-
-def patched_generate_dot(self, name, urls={}, env=None,
-                         graph_attrs={}, node_attrs={}, edge_attrs={}):
-    # Make a new mapping dictionary that uses class full names by importing each
-    # class documented name
-    fullname_urls = {self.class_name(try_import(name), 0, None): url
-                     for name, url in urls.items() if try_import(name) is not None}
-    return old_generate_dot(self, name, urls=fullname_urls, env=env,
-                            graph_attrs=graph_attrs, node_attrs=node_attrs, edge_attrs=edge_attrs)
+if SPHINX_LT_8_2:
+    def patched_generate_dot(self, name, urls={}, env=None,
+                             graph_attrs={}, node_attrs={}, edge_attrs={}):
+        # Make a new mapping dictionary that uses class full names by importing each
+        # class documented name
+        fullname_urls = {self.class_name(try_import(name), 0, None): url
+                         for name, url in urls.items() if try_import(name) is not None}
+        return old_generate_dot(self, name, urls=fullname_urls, env=env,
+                                graph_attrs=graph_attrs, node_attrs=node_attrs, edge_attrs=edge_attrs)
+else:
+    def patched_generate_dot(self, name, urls={}, config=None,
+                             graph_attrs={}, node_attrs={}, edge_attrs={}):
+        # Make a new mapping dictionary that uses class full names by importing each
+        # class documented name
+        fullname_urls = {self.class_name(try_import(name), 0, None): url
+                         for name, url in urls.items() if try_import(name) is not None}
+        return old_generate_dot(self, name, urls=fullname_urls, config=config,
+                                graph_attrs=graph_attrs, node_attrs=node_attrs, edge_attrs=edge_attrs)
 
 
 InheritanceGraph.generate_dot = patched_generate_dot
@@ -302,7 +327,8 @@ def process_automodsumm_generation(app):
                 lines, sfn, app=app, builder=app.builder,
                 base_path=app.srcdir,
                 inherited_members=app.config.automodsumm_inherited_members,
-                included_members=app.config.automodsumm_included_members)
+                included_members=app.config.automodsumm_included_members,
+                properties_are_attributes=app.config.automodsumm_properties_are_attributes)
 
 
 # _automodsummrex = re.compile(r'^(\s*)\.\. automodsumm::\s*([A-Za-z0-9_.]+)\s*'
@@ -380,12 +406,12 @@ def automodsumm_to_autosummary_lines(fn, app):
                                                       opssecs, remainders)):
         allindent = i1 + ('    ' if i2 is None else i2)
 
-        # filter out functions-only, classes-only, and ariables-only
+        # filter out functions-only, classes-only, variables-only, and sort
         # options if present.
         oplines = ops.split('\n')
         toskip = []
         allowedpkgnms = []
-        funcsonly = clssonly = varsonly = False
+        funcsonly = clssonly = varsonly = sort = False
         for i, ln in reversed(list(enumerate(oplines))):
             if ':functions-only:' in ln:
                 funcsonly = True
@@ -402,6 +428,9 @@ def automodsumm_to_autosummary_lines(fn, app):
             if ':allowed-package-names:' in ln:
                 allowedpkgnms.extend(_str_list_converter(ln.replace(':allowed-package-names:', '')))
                 del oplines[i]
+            if ':sort:' in ln:
+                sort = True
+                del oplines[i]
         if [funcsonly, clssonly, varsonly].count(True) > 1:
             msg = ('Defined more than one of functions-only, classes-only, '
                    'and variables-only.  Skipping this directive.')
@@ -419,7 +448,7 @@ def automodsumm_to_autosummary_lines(fn, app):
         newlines.extend(oplines)
 
         ols = True if len(allowedpkgnms) == 0 else allowedpkgnms
-        for nm, fqn, obj in zip(*find_mod_objs(modnm, onlylocals=ols)):
+        for nm, fqn, obj in zip(*find_mod_objs(modnm, onlylocals=ols, sort=sort)):
             if nm in toskip:
                 continue
             if funcsonly and not inspect.isroutine(obj):
@@ -440,7 +469,8 @@ def generate_automodsumm_docs(lines, srcfn, app=None, suffix='.rst',
                               base_path=None, builder=None,
                               template_dir=None,
                               inherited_members=False,
-                              included_members=('__init__', '__call__')):
+                              included_members=('__init__', '__call__'),
+                              *, properties_are_attributes=True):
     """
     This function is adapted from
     `sphinx.ext.autosummary.generate.generate_autosummmary_docs` to
@@ -450,13 +480,14 @@ def generate_automodsumm_docs(lines, srcfn, app=None, suffix='.rst',
     """
 
     from sphinx.jinja2glue import BuiltinTemplateLoader
-    from sphinx.ext.autosummary import import_by_name, get_documenter
+    from sphinx.ext.autosummary import import_by_name
     from sphinx.util.osutil import ensuredir
     from sphinx.util.inspect import safe_getattr
     from jinja2 import FileSystemLoader, TemplateNotFound
     from jinja2.sandbox import SandboxedEnvironment
 
     from .utils import find_autosummary_in_lines_for_automodsumm as find_autosummary_in_lines
+    from .utils import get_object_type
 
     # Create our own templating environment - here we use Astropy's
     # templates rather than the default autosummary templates, in order to
@@ -505,7 +536,7 @@ def generate_automodsumm_docs(lines, srcfn, app=None, suffix='.rst',
         try:
             import_by_name_values = import_by_name(name)
         except ImportError as e:
-            logger.warning('[automodsumm] failed to import %r: %s' % (name, e))
+            logger.warning('[automodsumm] failed to import {!r}: {}'.format(name, e))
             continue
 
         # if block to accommodate Sphinx's v1.2.2 and v1.2.3 respectively
@@ -522,18 +553,16 @@ def generate_automodsumm_docs(lines, srcfn, app=None, suffix='.rst',
 
         new_files.append(fn)
 
-        f = open(fn, 'w', encoding='utf8')
+        with open(fn, 'w', encoding='utf8') as f:
 
-        try:
-
-            doc = get_documenter(app, obj, parent)
+            obj_type = get_object_type(app, obj, parent)
 
             if template_name is not None:
                 template = template_env.get_template(template_name)
             else:
                 tmplstr = 'autosummary_core/%s.rst'
                 try:
-                    template = template_env.get_template(tmplstr % doc.objtype)
+                    template = template_env.get_template(tmplstr % obj_type)
                 except TemplateNotFound:
                     template = template_env.get_template(tmplstr % 'base')
 
@@ -544,10 +573,10 @@ def generate_automodsumm_docs(lines, srcfn, app=None, suffix='.rst',
                 items = []
                 for name in dir(obj):
                     try:
-                        documenter = get_documenter(app, safe_getattr(obj, name), obj)
+                        obj_type = get_object_type(app, safe_getattr(obj, name), obj)
                     except AttributeError:
                         continue
-                    if typ is None or documenter.objtype == typ:
+                    if typ is None or obj_type == typ:
                         items.append(name)
                 public = [x for x in items
                           if x in include_public or not x.startswith('_')]
@@ -568,24 +597,35 @@ def generate_automodsumm_docs(lines, srcfn, app=None, suffix='.rst',
                 else:
                     names = getattr(obj, '__dict__').keys()
 
+                    # add dataclass_field names for dataclass classes
+                    if dataclasses.is_dataclass(obj):
+                        dataclass_fieldnames = getattr(obj, '__dataclass_fields__').keys()
+                        names = list(set(list(names) + list(dataclass_fieldnames)))
+
                 for name in names:
                     try:
-                        documenter = get_documenter(app, safe_getattr(obj, name), obj)
+                        obj_type = get_object_type(app, safe_getattr(obj, name), obj)
                     except AttributeError:
-                        continue
-                    if typ is None or documenter.objtype == typ:
-                        items.append(name)
-                    elif typ == 'attribute' and documenter.objtype == 'property':
-                        # In Sphinx 2.0 and above, properties have a separate
-                        # objtype, but we treat them the same here.
+                        # for dataclasses try to get the attribute from the __dataclass_fields__
+                        if dataclasses.is_dataclass(obj):
+                            try:
+                                attr = obj.__dataclass_fields__[name]
+                                obj_type = get_object_type(app, attr, obj)
+                            except KeyError:
+                                continue
+                    if typ is None or obj_type == typ:
                         items.append(name)
+                    # elif typ == 'attribute' and obj_type == 'property':
+                    #     # In Sphinx 2.0 and above, properties have a separate
+                    #     # object type, but we treat them the same here.
+                    #     items.append(name)
                 public = [x for x in items
                           if x in include_public or not x.startswith('_')]
                 return public, items
 
             ns = {}
 
-            if doc.objtype == 'module':
+            if obj_type == 'module':
                 ns['members'] = get_members_mod(obj, None)
                 ns['functions'], ns['all_functions'] = \
                     get_members_mod(obj, 'function')
@@ -593,7 +633,7 @@ def generate_automodsumm_docs(lines, srcfn, app=None, suffix='.rst',
                     get_members_mod(obj, 'class')
                 ns['exceptions'], ns['all_exceptions'] = \
                     get_members_mod(obj, 'exception')
-            elif doc.objtype == 'class':
+            elif obj_type == 'class':
                 if inherited_mem is not None:
                     # option set in this specifc directive
                     include_base = inherited_mem
@@ -609,11 +649,20 @@ def generate_automodsumm_docs(lines, srcfn, app=None, suffix='.rst',
                 ns['attributes'], ns['all_attributes'] = \
                     get_members_class(obj, 'attribute',
                                       include_base=include_base)
+                public_properties, all_properties = \
+                    get_members_class(obj, 'property',
+                                      include_base=include_base)
+                if properties_are_attributes:
+                    ns['attributes'].extend(public_properties)
+                    ns['all_attributes'].extend(all_properties)
+                else:
+                    ns['properties'] = public_properties
+                    ns['all_properties'] = all_properties
                 ns['methods'].sort()
                 ns['attributes'].sort()
 
             parts = name.split('.')
-            if doc.objtype in ('method', 'attribute'):
+            if obj_type in ('method', 'attribute'):
                 mod_name = '.'.join(parts[:-2])
                 cls_name = parts[-2]
                 obj_name = '.'.join(parts[-2:])
@@ -627,7 +676,7 @@ def generate_automodsumm_docs(lines, srcfn, app=None, suffix='.rst',
             ns['objname'] = obj_name
             ns['name'] = parts[-1]
 
-            ns['objtype'] = doc.objtype
+            ns['objtype'] = obj_type
             ns['underline'] = len(obj_name) * '='
 
             # We now check whether a file for reference footnotes exists for
@@ -662,8 +711,6 @@ def generate_automodsumm_docs(lines, srcfn, app=None, suffix='.rst',
 
             rendered = template.render(**ns)
             f.write(cleanup_whitespace(rendered))
-        finally:
-            f.close()
 
 
 def setup(app):
@@ -685,6 +732,7 @@ def setup(app):
     app.add_config_value('automodsumm_inherited_members', False, 'env')
     app.add_config_value(
         'automodsumm_included_members', ['__init__', '__call__'], 'env')
+    app.add_config_value('automodsumm_properties_are_attributes', True, 'env')
 
     return {'parallel_read_safe': True,
             'parallel_write_safe': True}
